#pragma once

#include <vector>
#include <memory>

#include "Keyboard.h"
#include "Screen.h"
#include "KeyboardEvent.h"
#include "PaintEvent.h"
#include "MouseEvent.h"
#include "WindowManager.h"
#include "UiEvent.h"
#include "AbstractApp.h"
#include "EventManager.h"

class AbstractDesktopApp;

class DesktopManager
{
public:

    DesktopManager(AbstractDesktopApp* application);

    ~DesktopManager();

    static std::unique_ptr<DesktopManager> Create(AbstractDesktopApp* application);

    void SetWindowManager(WindowManagerUPtr windowManager);

    AbstractApp* getMainApp() const;

    WindowManager* GetWindowManager() const;

    Keyboard* GetKeyboard() const;

    void AddScreen(ScreenPtr screen);

    mt::Screen* GetDefaultScreen() const;

    bool IsModified() const;

    void SetIsModified(bool modified);

    void SetKeyboard(KeyboardUPtr keyboard);

    void OnUiEvent(UiEventUPtr event);

    void OnKeyboardEvent(const KeyboardEvent* keyboardEvent);

    void OnMouseEvent(const MouseEvent* mouseEvent);

    void OnPaintEvent(const PaintEvent* paintEvent);

    void ClearEvents();

private:
    std::vector<ScreenPtr> mScreens;
    WindowManagerUPtr mWindowManager;
    KeyboardUPtr mKeyboard;
    EventManagerUPtr mEventManager;
    AbstractDesktopApp* mUiApplication;
    bool mModified;
};

using DesktopManagerUPtr = std::unique_ptr<DesktopManager>;
